@@ -8,6 +8,7 @@ package Net::Async::WebSocket::Client;
 use strict;
 use warnings;
 use base qw( Net::Async::WebSocket::Protocol );
+use 5.010;  # //
 
 IO::Async::Notifier->VERSION( '0.63' ); # ->adopt_future
 
@@ -40,7 +41,7 @@ BEGIN {
    } unless URI::wss->can( "resource_name" );
 }
 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 use Protocol::WebSocket::Handshake::Client;
 
@@ -79,6 +80,15 @@ server to establish a WebSocket connection for passing frames.
 
 =cut
 
+sub new
+{
+   my $class = shift;
+   return $class->SUPER::new(
+      masked => 1,
+      @_,
+   );
+}
+
 =head1 METHODS
 
 The following methods documented with a trailing call to C<< ->get >> return
@@ -160,12 +170,13 @@ sub connect
    if( my $url = $params{url} ) {
       $url = URI->new( $url ) unless blessed $url and $url->isa( "URI" );
 
-      $params{host} //= $url->host;
+      $params{host}    //= $url->host;
       $params{service} //= $url->port;
 
       if( $url->secure ) {
          require IO::Async::SSL;
          push @{ $params{extensions} }, qw( SSL );
+         $params{SSL_hostname} //= $url->host;
       }
    }
 
