@@ -11,9 +11,10 @@ use base qw( IO::Async::Stream );
 
 use Carp;
 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 use Protocol::WebSocket::Frame;
+use Sub::Util 1.40 qw( set_subname );
 
 my %FRAMETYPES = (
    1 => "text",
@@ -65,7 +66,7 @@ A synonym for C<on_text_frame>, provided for backward compatibility.
 
 This may be removed in a later version.
 
-=head2 on_binary_frame, on_ping_frame, on_pong_frame
+=head2 on_binary_frame, on_ping_frame, on_pong_frame, on_close_frame
 
    $self->on_..._frame( $bytes )
    $on_..._frame->( $self, $bytes )
@@ -102,12 +103,16 @@ The following named parameters may be passed to C<new> or C<configure>:
 
 =item on_text_frame => CODE
 
-=item on_binary_frame, on_ping_frame, on_pong_frame => CODE
+=item on_binary_frame, on_ping_frame, on_pong_frame, on_close_frame => CODE
 
 =item on_raw_frame => CODE
 
 CODE references for event handlers.
 
+=item masked => BOOL
+
+Whether frames constructed and sent by this instance will be masked.
+
 =back
 
 =cut
@@ -117,7 +122,7 @@ sub configure
    my $self = shift;
    my %params = @_;
 
-   foreach (qw( on_frame on_raw_frame ), @ON_TYPE_FRAMES ) {
+   foreach (qw( on_frame on_raw_frame masked ), @ON_TYPE_FRAMES ) {
       $self->{$_} = delete $params{$_} if exists $params{$_};
    }
 
@@ -137,11 +142,12 @@ sub on_read
       my $type = $FRAMETYPES{$framebuffer->opcode};
       $self->debug_printf( "FRAME $type" );
 
-      $self->maybe_invoke_event( "on_${type}_frame" =>
-         $type eq "text" ? Encode::decode_utf8( $bytes ) : $bytes )
-            or $self->maybe_invoke_event( on_raw_frame => $framebuffer, $bytes );
+      my $text = $framebuffer->is_text ? Encode::decode_utf8( $bytes ) : undef;
 
-      $self->maybe_invoke_event( on_frame => Encode::decode_utf8( $bytes ) ) if $framebuffer->is_text;
+      $self->maybe_invoke_event( "on_${type}_frame" => $text // $bytes )
+         or $self->maybe_invoke_event( on_raw_frame => $framebuffer, $bytes );
+
+      $self->maybe_invoke_event( on_frame => $text ) if $framebuffer->is_text;
    }
 
    return 0;
@@ -176,7 +182,7 @@ sub send_frame
 
 =head2 send_text_frame
 
-   $self->send_text_frame( $text, %params )
+   $self->send_text_frame( $text, %params )->get
 
 Sends a text frame to the peer. The given string will be treated as a Unicode
 character string, and sent as UTF-8 encoded bytes.
@@ -186,11 +192,13 @@ L<IO::Async::Stream/write> call.
 
 =head2 send_I<TYPE>_frame
 
-   $self->send_binary_frame( $bytes, %params )
+   $self->send_binary_frame( $bytes, %params )->get
 
-   $self->send_ping_frame( $bytes, %params )
+   $self->send_ping_frame( $bytes, %params )->get
 
-   $self->send_pong_frame( $bytes, %params )
+   $self->send_pong_frame( $bytes, %params )->get
+
+   $self->send_close_frame( $bytes, %params )->get
 
 Sends a frame of the given type to the peer.
 
@@ -206,42 +214,34 @@ sub send_text_frame
 
    # Protocol::WebSocket::Frame will UTF-8 encode this for us
    $self->write(
-      Protocol::WebSocket::Frame->new( type => "text", buffer => $text )->to_bytes,
+      Protocol::WebSocket::Frame->new(
+         type   => "text",
+         buffer => $text,
+         masked => $self->{masked},
+      )->to_bytes,
       %params
    );
 }
 
-sub send_binary_frame
-{
-   my $self = shift;
-   my ( $bytes, %params ) = @_;
+foreach my $type ( values %FRAMETYPES ) {
+   next if $type eq "text";
+   my $method = "send_${type}_frame";
+   my $code = sub {
+      my $self = shift;
+      my ( $bytes, %params ) = @_;
 
-   $self->write(
-      Protocol::WebSocket::Frame->new( type => "binary", buffer => $bytes )->to_bytes,
-      %params
-   );
-}
+      $self->write(
+         Protocol::WebSocket::Frame->new(
+            type   => $type,
+            buffer => $bytes,
+            masked => $self->{masked},
+         )->to_bytes,
+         %params
+      );
+   };
 
-sub send_ping_frame
-{
-   my $self = shift;
-   my ( $bytes, %params ) = @_;
-
-   $self->write(
-      Protocol::WebSocket::Frame->new( type => "ping", buffer => $bytes )->to_bytes,
-      %params
-   );
-}
-
-sub send_pong_frame
-{
-   my $self = shift;
-   my ( $bytes, %params ) = @_;
-
-   $self->write(
-      Protocol::WebSocket::Frame->new( type => "pong", buffer => $bytes )->to_bytes,
-      %params
-   );
+   no strict 'refs';
+   *$method = set_subname $method => $code;
 }
 
 =head1 AUTHOR
